@@ -15,7 +15,6 @@ import os
 import socket
 
 import mock
-import os_brick
 from oslo_concurrency import processutils
 from oslo_config import cfg
 
@@ -26,10 +25,8 @@ from cinder.objects import fields
 from cinder.tests import fake_driver
 from cinder.tests.unit.brick import fake_lvm
 from cinder.tests.unit import fake_constants as fake
-from cinder.tests.unit import test_volume
-from cinder.tests.unit.test_volume import fake_opt
 from cinder.tests.unit import utils as tests_utils
-from cinder import utils
+from cinder.tests.unit.volume import test_driver
 from cinder.volume import configuration as conf
 from cinder.volume.drivers import lvm
 import cinder.volume.utils
@@ -37,16 +34,20 @@ from cinder.volume import utils as volutils
 
 CONF = cfg.CONF
 
+fake_opt = [
+    cfg.StrOpt('fake_opt1', default='fake', help='fake opts')
+]
+
 
 @ddt.ddt
-class LVMVolumeDriverTestCase(test_volume.DriverTestCase):
+class LVMVolumeDriverTestCase(test_driver.BaseDriverTestCase):
     """Test case for VolumeDriver"""
     driver_name = "cinder.volume.drivers.lvm.LVMVolumeDriver"
     FAKE_VOLUME = {'name': 'test1',
                    'id': 'test1'}
 
     @mock.patch.object(os.path, 'exists', return_value=True)
-    @mock.patch.object(fake_driver.FakeISCSIDriver, 'create_export')
+    @mock.patch.object(fake_driver.FakeLoggingVolumeDriver, 'create_export')
     def test_delete_volume_invalid_parameter(self, _mock_create_export,
                                              mock_exists):
         self.configuration.volume_clear = 'zero'
@@ -59,7 +60,7 @@ class LVMVolumeDriverTestCase(test_volume.DriverTestCase):
                           self.FAKE_VOLUME)
 
     @mock.patch.object(os.path, 'exists', return_value=False)
-    @mock.patch.object(fake_driver.FakeISCSIDriver, 'create_export')
+    @mock.patch.object(fake_driver.FakeLoggingVolumeDriver, 'create_export')
     def test_delete_volume_bad_path(self, _mock_create_export, mock_exists):
         self.configuration.volume_clear = 'zero'
         self.configuration.volume_clear_size = 0
@@ -74,7 +75,7 @@ class LVMVolumeDriverTestCase(test_volume.DriverTestCase):
 
     @mock.patch.object(volutils, 'clear_volume')
     @mock.patch.object(volutils, 'copy_volume')
-    @mock.patch.object(fake_driver.FakeISCSIDriver, 'create_export')
+    @mock.patch.object(fake_driver.FakeLoggingVolumeDriver, 'create_export')
     def test_delete_volume_thinlvm_snap(self, _mock_create_export,
                                         mock_copy, mock_clear):
         vg_obj = fake_lvm.FakeBrickLVM('cinder-volumes',
@@ -84,7 +85,7 @@ class LVMVolumeDriverTestCase(test_volume.DriverTestCase):
         self.configuration.volume_clear = 'zero'
         self.configuration.volume_clear_size = 0
         self.configuration.lvm_type = 'thin'
-        self.configuration.iscsi_helper = 'tgtadm'
+        self.configuration.target_helper = 'tgtadm'
         lvm_driver = lvm.LVMVolumeDriver(configuration=self.configuration,
                                          vg_obj=vg_obj, db=db)
 
@@ -97,11 +98,13 @@ class LVMVolumeDriverTestCase(test_volume.DriverTestCase):
 
     @mock.patch.object(volutils, 'get_all_volume_groups',
                        return_value=[{'name': 'cinder-volumes'}])
-    def test_check_for_setup_error(self, vgs):
+    @mock.patch('cinder.brick.local_dev.lvm.LVM.get_lvm_version',
+                return_value=(2, 2, 100))
+    def test_check_for_setup_error(self, _mock_get_version, vgs):
         vg_obj = fake_lvm.FakeBrickLVM('cinder-volumes',
                                        False,
                                        None,
-                                       'default')
+                                       'auto')
 
         configuration = conf.Configuration(fake_opt, 'fake_group')
         lvm_driver = lvm.LVMVolumeDriver(configuration=configuration,
@@ -132,41 +135,6 @@ class LVMVolumeDriverTestCase(test_volume.DriverTestCase):
 
         lvm_driver.check_for_setup_error()
 
-    @mock.patch.object(utils, 'temporary_chown')
-    @mock.patch('six.moves.builtins.open')
-    @mock.patch.object(os_brick.initiator.connector,
-                       'get_connector_properties')
-    @mock.patch.object(db.sqlalchemy.api, 'volume_get')
-    def test_backup_volume(self, mock_volume_get,
-                           mock_get_connector_properties,
-                           mock_file_open,
-                           mock_temporary_chown):
-        vol = tests_utils.create_volume(self.context)
-        self.context.user_id = fake.USER_ID
-        self.context.project_id = fake.PROJECT_ID
-        backup_obj = tests_utils.create_backup(self.context,
-                                               vol['id'])
-
-        properties = {}
-        attach_info = {'device': {'path': '/dev/null'}}
-        backup_service = mock.Mock()
-
-        self.volume.driver._detach_volume = mock.MagicMock()
-        self.volume.driver._attach_volume = mock.MagicMock()
-        self.volume.driver.terminate_connection = mock.MagicMock()
-
-        mock_volume_get.return_value = vol
-        mock_get_connector_properties.return_value = properties
-        f = mock_file_open.return_value = open('/dev/null', 'rb')
-
-        backup_service.backup(backup_obj, f, None)
-        self.volume.driver._attach_volume.return_value = attach_info
-
-        self.volume.driver.backup_volume(self.context, backup_obj,
-                                         backup_service)
-
-        mock_volume_get.assert_called_with(self.context, vol['id'])
-
     def test_retype_volume(self):
         vol = tests_utils.create_volume(self.context)
         new_type = fake.VOLUME_TYPE_ID
@@ -213,52 +181,6 @@ class LVMVolumeDriverTestCase(test_volume.DriverTestCase):
                               'provider_location': fake_provider},
                              update)
 
-    @mock.patch.object(utils, 'temporary_chown')
-    @mock.patch('six.moves.builtins.open')
-    @mock.patch.object(os_brick.initiator.connector,
-                       'get_connector_properties')
-    @mock.patch.object(db.sqlalchemy.api, 'volume_get')
-    def test_backup_volume_inuse(self, mock_volume_get,
-                                 mock_get_connector_properties,
-                                 mock_file_open,
-                                 mock_temporary_chown):
-
-        vol = tests_utils.create_volume(self.context,
-                                        status='backing-up',
-                                        previous_status='in-use')
-        self.context.user_id = fake.USER_ID
-        self.context.project_id = fake.PROJECT_ID
-
-        mock_volume_get.return_value = vol
-        temp_snapshot = tests_utils.create_snapshot(self.context, vol['id'])
-        backup_obj = tests_utils.create_backup(self.context,
-                                               vol['id'])
-        properties = {}
-        attach_info = {'device': {'path': '/dev/null'}}
-        backup_service = mock.Mock()
-
-        self.volume.driver._detach_volume = mock.MagicMock()
-        self.volume.driver._attach_volume = mock.MagicMock()
-        self.volume.driver.terminate_connection = mock.MagicMock()
-        self.volume.driver._create_temp_snapshot = mock.MagicMock()
-        self.volume.driver._delete_temp_snapshot = mock.MagicMock()
-
-        mock_get_connector_properties.return_value = properties
-        f = mock_file_open.return_value = open('/dev/null', 'rb')
-
-        backup_service.backup(backup_obj, f, None)
-        self.volume.driver._attach_volume.return_value = attach_info
-        self.volume.driver._create_temp_snapshot.return_value = temp_snapshot
-
-        self.volume.driver.backup_volume(self.context, backup_obj,
-                                         backup_service)
-
-        mock_volume_get.assert_called_with(self.context, vol['id'])
-        self.volume.driver._create_temp_snapshot.assert_called_once_with(
-            self.context, vol)
-        self.volume.driver._delete_temp_snapshot.assert_called_once_with(
-            self.context, temp_snapshot)
-
     def test_create_volume_from_snapshot_none_sparse(self):
 
         with mock.patch.object(self.volume.driver, 'vg'), \
@@ -290,9 +212,7 @@ class LVMVolumeDriverTestCase(test_volume.DriverTestCase):
         lvm_driver = lvm.LVMVolumeDriver(configuration=self.configuration,
                                          db=db)
 
-        with mock.patch.object(lvm_driver, 'vg'), \
-                mock.patch.object(lvm_driver, '_create_volume'), \
-                mock.patch.object(volutils, 'copy_volume') as mock_copy:
+        with mock.patch.object(lvm_driver, 'vg'):
 
             # Test case for thin LVM
             lvm_driver._sparse_copy_volume = True
@@ -303,16 +223,25 @@ class LVMVolumeDriverTestCase(test_volume.DriverTestCase):
             lvm_driver.create_volume_from_snapshot(dst_volume,
                                                    snapshot_ref)
 
-            volume_path = lvm_driver.local_path(dst_volume)
-            snapshot_path = lvm_driver.local_path(snapshot_ref)
-            volume_size = 1024
-            block_size = '1M'
-            mock_copy.assert_called_with(snapshot_path,
-                                         volume_path,
-                                         volume_size,
-                                         block_size,
-                                         execute=lvm_driver._execute,
-                                         sparse=True)
+    def test_create_volume_from_snapshot_sparse_extend(self):
+
+        self.configuration.lvm_type = 'thin'
+        lvm_driver = lvm.LVMVolumeDriver(configuration=self.configuration,
+                                         db=db)
+
+        with mock.patch.object(lvm_driver, 'vg'), \
+                mock.patch.object(lvm_driver, 'extend_volume') as mock_extend:
+
+            # Test case for thin LVM
+            lvm_driver._sparse_copy_volume = True
+            src_volume = tests_utils.create_volume(self.context)
+            snapshot_ref = tests_utils.create_snapshot(self.context,
+                                                       src_volume['id'])
+            dst_volume = tests_utils.create_volume(self.context)
+            dst_volume['size'] = snapshot_ref['volume_size'] + 1
+            lvm_driver.create_volume_from_snapshot(dst_volume,
+                                                   snapshot_ref)
+            mock_extend.assert_called_with(dst_volume, dst_volume['size'])
 
     @mock.patch.object(cinder.volume.utils, 'get_all_volume_groups',
                        return_value=[{'name': 'cinder-volumes'}])
@@ -362,6 +291,10 @@ class LVMVolumeDriverTestCase(test_volume.DriverTestCase):
 
     @mock.patch.object(cinder.volume.utils, 'get_all_volume_groups',
                        return_value=[{'name': 'cinder-volumes'}])
+    @mock.patch('cinder.brick.local_dev.lvm.LVM.get_lv_info')
+    @mock.patch('cinder.brick.local_dev.lvm.LVM.activate_lv')
+    @mock.patch('cinder.brick.local_dev.lvm.LVM.'
+                'supports_lvchange_ignoreskipactivation')
     @mock.patch('cinder.brick.local_dev.lvm.LVM.update_volume_group_info')
     @mock.patch('cinder.brick.local_dev.lvm.LVM.get_all_physical_volumes')
     @mock.patch('cinder.brick.local_dev.lvm.LVM.supports_thin_provisioning',
@@ -378,6 +311,10 @@ class LVMVolumeDriverTestCase(test_volume.DriverTestCase):
 
     @mock.patch.object(cinder.volume.utils, 'get_all_volume_groups',
                        return_value=[{'name': 'cinder-volumes'}])
+    @mock.patch('cinder.brick.local_dev.lvm.LVM.get_lv_info')
+    @mock.patch('cinder.brick.local_dev.lvm.LVM.activate_lv')
+    @mock.patch('cinder.brick.local_dev.lvm.LVM.'
+                'supports_lvchange_ignoreskipactivation')
     @mock.patch('cinder.brick.local_dev.lvm.LVM.update_volume_group_info')
     @mock.patch('cinder.brick.local_dev.lvm.LVM.get_all_physical_volumes')
     @mock.patch('cinder.brick.local_dev.lvm.LVM.get_volume')
@@ -546,6 +483,7 @@ class LVMVolumeDriverTestCase(test_volume.DriverTestCase):
                                                           False,
                                                           None,
                                                           'default')
+            mock_execute.return_value = ("mock_outs", "mock_errs")
             moved, model_update = \
                 self.volume.driver.migrate_volume(self.context, vol, host)
             self.assertTrue(moved)
@@ -594,6 +532,7 @@ class LVMVolumeDriverTestCase(test_volume.DriverTestCase):
                                                   None,
                                                   'default')
             lvm_driver._sparse_copy_volume = True
+            mock_execute.return_value = ("mock_outs", "mock_errs")
             moved, model_update = \
                 lvm_driver.migrate_volume(self.context, vol, host)
             self.assertTrue(moved)
@@ -763,6 +702,49 @@ class LVMVolumeDriverTestCase(test_volume.DriverTestCase):
                 self.volume.driver.manage_existing_snapshot_get_size,
                 snp, ref)
 
+    def test_revert_snapshot(self):
+        self._setup_stubs_for_manage_existing()
+        self.configuration.lvm_type = 'auto'
+        fake_volume = tests_utils.create_volume(self.context,
+                                                display_name='fake_volume')
+        fake_snapshot = tests_utils.create_snapshot(
+            self.context, fake_volume.id)
+
+        with mock.patch.object(self.volume.driver.vg,
+                               'revert') as mock_revert,\
+                mock.patch.object(self.volume.driver.vg,
+                                  'create_lv_snapshot') as mock_create,\
+                mock.patch.object(self.volume.driver.vg,
+                                  'deactivate_lv') as mock_deactive,\
+                mock.patch.object(self.volume.driver.vg,
+                                  'activate_lv') as mock_active:
+            self.volume.driver.revert_to_snapshot(self.context,
+                                                  fake_volume,
+                                                  fake_snapshot)
+            mock_revert.assert_called_once_with(
+                self.volume.driver._escape_snapshot(fake_snapshot.name))
+            mock_deactive.assert_called_once_with(fake_volume.name)
+            mock_active.assert_called_once_with(fake_volume.name)
+            mock_create.assert_called_once_with(
+                self.volume.driver._escape_snapshot(fake_snapshot.name),
+                fake_volume.name, self.configuration.lvm_type)
+
+    def test_revert_thin_snapshot(self):
+
+        configuration = conf.Configuration(fake_opt, 'fake_group')
+        configuration.lvm_type = 'thin'
+        lvm_driver = lvm.LVMVolumeDriver(configuration=configuration,
+                                         db=db)
+        fake_volume = tests_utils.create_volume(self.context,
+                                                display_name='fake_volume')
+        fake_snapshot = tests_utils.create_snapshot(
+            self.context, fake_volume.id)
+
+        self.assertRaises(NotImplementedError,
+                          lvm_driver.revert_to_snapshot,
+                          self.context, fake_volume,
+                          fake_snapshot)
+
     def test_lvm_manage_existing_snapshot_bad_size(self):
         """Make sure correct exception on bad size returned from LVM.
 
@@ -805,11 +787,12 @@ class LVMVolumeDriverTestCase(test_volume.DriverTestCase):
         res = self.volume.driver.get_manageable_volumes(cinder_vols, None,
                                                         1000, 0,
                                                         ['size'], ['asc'])
-        exp = [{'size': 2, 'reason_not_safe': None, 'extra_info': None,
+        exp = [{'size': 2, 'reason_not_safe': 'already managed',
+                'extra_info': None,
                 'reference': {'source-name':
                               'volume-00000000-0000-0000-0000-000000000000'},
                 'cinder_id': '00000000-0000-0000-0000-000000000000',
-                'safe_to_manage': False, 'reason_not_safe': 'already managed'},
+                'safe_to_manage': False},
                {'size': 3, 'reason_not_safe': 'volume in use',
                 'reference': {'source-name':
                               'volume-00000000-0000-0000-0000-000000000001'},
@@ -866,37 +849,17 @@ class LVMVolumeDriverTestCase(test_volume.DriverTestCase):
                 'extra_info': None}]
         self.assertEqual(exp, res)
 
-    # Global setting, LVM setting, expected outcome
-    @ddt.data((10.0, 2.0, 2.0))
-    @ddt.data((10.0, None, 10.0))
-    @ddt.unpack
-    def test_lvm_max_over_subscription_ratio(self,
-                                             global_value,
-                                             lvm_value,
-                                             expected_value):
-        configuration = conf.Configuration(fake_opt, 'fake_group')
-        configuration.max_over_subscription_ratio = global_value
-        configuration.lvm_max_over_subscription_ratio = lvm_value
 
-        fake_vg = mock.Mock(fake_lvm.FakeBrickLVM('cinder-volumes', False,
-                                                  None, 'default'))
-        lvm_driver = lvm.LVMVolumeDriver(configuration=configuration,
-                                         vg_obj=fake_vg, db=db)
-
-        self.assertEqual(expected_value,
-                         lvm_driver.configuration.max_over_subscription_ratio)
-
-
-class LVMISCSITestCase(test_volume.DriverTestCase):
+class LVMISCSITestCase(test_driver.BaseDriverTestCase):
     """Test Case for LVMISCSIDriver"""
     driver_name = "cinder.volume.drivers.lvm.LVMVolumeDriver"
 
     def setUp(self):
         super(LVMISCSITestCase, self).setUp()
         self.configuration = mock.Mock(conf.Configuration)
-        self.configuration.iscsi_target_prefix = 'iqn.2010-10.org.openstack:'
-        self.configuration.iscsi_ip_address = '0.0.0.0'
-        self.configuration.iscsi_port = 3260
+        self.configuration.target_prefix = 'iqn.2010-10.org.openstack:'
+        self.configuration.target_ip_address = '0.0.0.0'
+        self.configuration.target_port = 3260
 
     def _attach_volume(self):
         """Attach volumes to an instance."""
@@ -905,7 +868,7 @@ class LVMISCSITestCase(test_volume.DriverTestCase):
             vol = {}
             vol['size'] = 0
             vol_ref = db.volume_create(self.context, vol)
-            self.volume.create_volume(self.context, vol_ref['id'])
+            self.volume.create_volume(self.context, vol_ref)
             vol_ref = db.volume_get(self.context, vol_ref['id'])
 
             # each volume has a different mountpoint
@@ -923,7 +886,7 @@ class LVMISCSITestCase(test_volume.DriverTestCase):
             cinder.volume.targets.tgt.TgtAdm(
                 configuration=self.configuration)
 
-        ret = ("%s dummy" % CONF.iscsi_ip_address, '')
+        ret = ("%s dummy" % CONF.target_ip_address, '')
         with mock.patch('cinder.utils.execute',
                         return_value=ret):
             volume = {"name": "dummy",
@@ -994,6 +957,7 @@ class LVMISCSITestCase(test_volume.DriverTestCase):
         # This value is set in check_for_setup_error.
         self.configuration = conf.Configuration(None)
         self.configuration.lvm_type = 'thin'
+        self.configuration.target_helper = 'lioadm'
         vg_obj = fake_lvm.FakeBrickLVM('cinder-volumes',
                                        False,
                                        None,
